// version du 31/07/04 OK ....
// PIC 12C508
// utilise le timer  1:8
// rajout de la fct inverseur sur GP4 
// rajout de la fct servo sur GP1
// copyright A. GLEYZES 2004 

#include <pic.h>

__CONFIG(INTRC | UNPROTECT); 	/* rq MCLRE et XDTE sont  0 */

// define locaux
// *************
#define S_moteur GP0
#define S_inverse GP4
#define S_servo GP1
#define S_LED GP2
#define E_BEC GP3
#define ON 1
#define OFF 0
#define E_emetteur GP5
#define Detecte 0


/******************************************************************************************
*
*	MODULE "UNE VOIE DE PLUS" :
*   =========================
*
*	permet de commander une voie en tout ou rien branche en parallle sur une autre 
*	voie proportionnelle.
*	Inclut une fonction BEC.
*	inclut un inverseur de servo
*
*	p7 : GP0 = sortie moteur ON/OFF 
*	p6 : GP1 = sortie servo tout ou rien
*	p5 : GP2 = sortie LED test
*	p4 : GP3 = entre BEC 
*	p3 : GP4 = sortie inverse
*	p2 : GP5 = entre metteur
*
*	GP0, 1 et 3 : pull up internes actifs
*
*	A la mise sous tension, la LED s'allume. Le boot demarre par un talonnage de la 
*	dure du neutre. Le manche doit donc tre laiss au neutre pendant 3 secondes.
*	Aprs calcul de la dure du neutre, le manche de profondeur (si c'est la voie utilise)
*	doit tre positionn au minimum (plein piqu) pendant 2 secondes, puis au maximum 
*	pendant deux secondes egalement (plein cabr) pour talonnage de la course. 
*	Lorsque cette manoeuvre est effectuee, la LED s'eteint.
*
*   Ensuite, toute action sur le manche de type mini+maxi dans un dlai trs bref
*	provoque alternativement la mise ON/mise OFF de la voie tout ou rien,
*	ainsi que le dplacement en tout ou rien d'un servo connect sur GP1.
*   Si la voie tout ou rien est utilise pour mettre ON un moteur et que la fonction BEC
*	est cable (GP3), alors, lorsque la batterie passe en dessous de 5,7 V, 
*	le moteur est coup. (GP0 = OFF). 
*	Une remise ON du moteur est possible (coup de manche mini+maxi) pour tirer le
* 	maximum des accus ...
*	Si le BEC n'est pas cabl, GP3 doit tre raccord au +5V.
*
*	Il faut noter que la fonction BEC n'a pas d'effet sur la sortie servo tout ou rien (GP1)
*
*	Le montage peut aussi servir d'inverseur de sens de rotation de servo.
*	le signal sur GP4 est l'inverse du signal servo en entree. L'inverseur est effectif
*	lorsque l' initialisation du neutre a t effectue. (3 secondes aprs mise ON)
*
*******************************************************************************************/

typedef union			/* utilis pour grer les dbordements du timer */
{
	struct 
	{
		unsigned char low;
		unsigned char high;
	} n_cpt ;
	int cpt ;
} compteur;




/* pas d'appels imbriqus ==> on peut passer par le stack en "fastcall" */
fastcall void						
Entrees (compteur *servo1)
{	
	do {					/* attente dbut impulsion */
	} while (E_emetteur == 0);

	/* ici impulsion servo ==> rinit variables */
	
	TMR0 = 155;				/* reset timer */
	do {					/* attente talon 100*8 usec */
	} while (TMR0);
	
	do {
	} while (E_emetteur == 1);			/* attente fin impulsion */
	(*servo1).cpt = TMR0 + 100;	/* rajout suite TMR0 */	
}



/* pas d'appels imbriqus ==> on peut passer par le stack en "fastcall" */
fastcall void						
Sorties (compteur consigne1)
{
	if (consigne1.n_cpt.low)
		{
			TMR0 =  (0xFF - consigne1.n_cpt.low) +1;	
			do {		
			} while (TMR0);
		}
	while ( consigne1.n_cpt.high--) 
	{
		TMR0 = 1;
		do {		
		} while (TMR0);
	} 
	

}					







/***********************/
/* programme principal */
/***********************/

main (void)
{
	unsigned char j;
	unsigned char i;
	unsigned char count;
	unsigned char step;
	int delta;

	int delta_min;		// seuil de declenchement bas
	int delta_max;		// seuil de declenchement haut
	int autorise;		// variable d'autorisation de mise ON/OFF
	int filtre_BEC;		// filtre de BEC

	compteur consigne;
	compteur neutreA;
	compteur duree_ESA;
	
	
	OPTION = GPWU | GPPU | T0SE | !PSA | 0x02 ; /* wake up disabled
							Pull up disabled
							Internal cycle clock
							High to low source edge
							Prescaler assigned to timer 
							prescaler 1:8 */

	TRIS = 0b101000 ; 		/* GP3 et GP5 en entre */
	
	GPIO = 0x00;			/* LED=0=OFF, sorties  0 */
	neutreA.cpt = 187;		/* dure neutre 1,5 ms soit 187,5 ticks */


/* calcul du neutre */
/********************/
	S_LED = ON;			/* LED ON */
	
	i = 3;				/* boucle 3 secondes filtre le neutre */
	do {						
		j = 50;			/* boucle 50 Hz */
		do { 
			
			Entrees (&duree_ESA);
			
			/* calcul et filtre du neutre */
			
			neutreA.cpt = neutreA.cpt + duree_ESA.cpt;
			neutreA.cpt = neutreA.cpt>>1;	
								
		} while (--j);

	} while (--i);
	

	
	delta_min = -15;
	delta_max = +15;
	delta = 15;
	step = 0;
	
/* calcul des seuils de declenchement min et max */
/*************************************************/
deb_calib :	
	
	Entrees (&duree_ESA);

// calcul du seuil min 

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
	
	
	if (consigne.cpt > -delta)  	// le manche n'a pas t abaiss 
	{
		goto fin;
	}	
	count = 0;
	step = 1;
	
deb_min :	
	
	Entrees (&duree_ESA);

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
		

	if (consigne.cpt < delta_min)	// le manche est plein piqu on cherche le minimum
	{
		delta_min = consigne.cpt;	
	}
	count++;
	if (count < 50)	// on attend la valeur min pendant 50 pulses soit 1 sec 
	{
		goto fin;
	}
	delta_min = delta_min+5; // on garde des marges ... 
	
	step = 2;
	
deb_calmax :
	
	
	Entrees (&duree_ESA);

// calcul du seuil max 

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
	
	
	if (consigne.cpt < delta)  	// le manche n'a pas t relev 
	{
		goto fin;
	}	
	count = 0;
	step = 3;
	
deb_max :	
	
	Entrees (&duree_ESA);

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
	

	if (consigne.cpt > delta_max)	// le manche est plein cabr 
	{
		delta_max = consigne.cpt;	
	}
	count++;
	if (count < 50)	// on attend la valeur max pendant 1 sec 
	{
		goto fin;
	}
	delta_max = delta_max-5; // on garde des marges ... 
	step = 4;	
	
	
/* boucle principale  */
/**********************/
	S_LED = OFF; 			/* fin de initialisations */
	i = 0;
	j = 0;
	autorise = 0;
	filtre_BEC = 0;
	delta = delta_min - 5 ; // pour que la voie vario tout ou rien soit init  0
	
debut :
	
	Entrees (&duree_ESA);

/* calcul de la consigne */
	
	consigne.cpt = duree_ESA.cpt - neutreA.cpt;

	if (consigne.cpt <= delta_min)	/* gestion manche bas */
	{
		j = 0;
		i++;
		if (i == 6)	// filtre autorisation on est reste 100ms au min
		{
			i = 0;
			autorise = 1;
		}
		
		goto fin;	/* attente prochaine impulsion */	
	}

	if (consigne.cpt < delta_max)	/* gestion resets */
	{
		i = 0;
		j ++;
		if (j == 16) 	// on n'est pas pass assez vite de min  max : plus de 300ms
		{
			j = 0;
			autorise = 0;
		}
		
		goto fin;	// attente prochaine impulsion 
	}
	
	if (consigne.cpt >= delta_max)	// gestion mise ON/off du moteur 
	{
		if (autorise == 1)	// il faut changer le moteur d'etat
		{
			S_moteur = !S_moteur;
			autorise = 0;
			// basculement de la consigne pour la voie servo tout ou rien (stock ds delta qui ne servait plus  rien)
			if (delta > 0) delta = delta_min - 5;
			else delta = delta_max + 5;
		}
		
		i = 0;
		j = 0;
		
		
		//goto fin;	// attente prochaine impulsion 
	}

fin :
	// gestion BEC
		if (E_BEC == Detecte)	// on a detecte une tension bat faible
			{
				filtre_BEC++;
				if (filtre_BEC == 20)	// il faut couper le jus
				{
					S_moteur = OFF;
					filtre_BEC = 0;
				}
			}
		else filtre_BEC = 0;
		
		
	// generation voie inverse
	duree_ESA.cpt = neutreA.cpt - consigne.cpt;
	/* appliquer la consigne inverse sur sortie */
	S_inverse = 1;
	Sorties (duree_ESA);
	S_inverse = 0;	
	
	switch (step) {
		case 0:	
 			goto deb_calib;
 	 		break;
 	 	case 1:	
 			goto deb_min;
 	 		break;
		case 2:
 			goto deb_calmax;		
			break;
		case 3:
 			goto deb_max;		
			break;
		
		
		default : step = 4;
		}
	
	
	// generation voie servo tout ou rien uniquement lorsque les initialisations sont faites
	duree_ESA.cpt = neutreA.cpt + delta;
	/* appliquer la consigne servo sur sortie */
	S_servo = 1;
	Sorties (duree_ESA);
	S_servo = 0;	
	
	do {
	} while (E_emetteur == 1);	/* attente fin impulsion servo le cas chant */
	goto debut;
}

