// version du 13/10/07   : test OK sur inverseur 
// V3.0
//  
// PIC 12F675
// 
// rajout inverseur et voie propor
// version du 14/02/03 
// PIC 12C508
// utilise le timer  1:8
// copyright A. GLEYZES 2003 

#include <pic.h>

__CONFIG(UNPROTECT & BORDIS & MCLRDIS & PWRTEN & WDTDIS & INTIO); // config pour 12F675
		/* 	Data et Pgm non protgs
			BrownOut-Rst disabled
			MmCLR disabled => GPIO3 : Port-In
			Power up timer enabled
			Watchdog disable
			Intern.Osc, GPIO4-5 : Port I-O */

// define locaux
// *************
#define S_moteur GPIO0
#define S_servo GPIO1
#define S_LED GPIO2
#define E_BEC GPIO3
#define S_inverse GPIO4
#define ON 1
#define OFF 0
#define E_emetteur GPIO5
#define Detecte 0


/******************************************************************************************
*
*	MODULE "UNE VOIE DE PLUS" :
*   =========================
*
*	permet de commander une voie en tout ou rien branche en parallle sur une autre 
*	voie proportionnelle.
*	Inclut une fonction BEC.
*
*	p7 : GPIO0 = sortie moteur ON/OFF (AC)
*	p6 : GPIO1 = sortie servo tout ou rien
*	p5 : GPIO2 = sortie LED test
*	p4 : GPIO3 = entre BEC 
*	p3 : GPIO4 = sortie inverse
*	p2 : GPIO5 = entre metteur
*
*
*	A la mise sous tension, la LED s'allume. Le boot demarre par un talonnage de la 
*	dure du neutre. Le manche doit donc tre laiss au neutre pendant 3 secondes.
*	Aprs calcul de la dure du neutre, le manche de profondeur (si c'est la voie utilise)
*	doit tre positionn au minimum (plein piqu) pendant 2 secondes, puis au maximum 
*	pendant deux secondes egalement (plein cabr) pour talonnage de la course. 
*	Lorsque cette manoeuvre est effectuee, la LED s'eteint.
*
*   Ensuite, toute action sur le manche de type mini+maxi dans un dlai trs bref
*	provoque alternativement la mise ON/mise OFF de la voie tout ou rien,
*	ainsi que le dplacement en tout ou rien d'un servo connect sur GP1.
*   Si la voie tout ou rien est utilise pour mettre ON un moteur et que la fonction BEC
*	est cable (GP3), alors, lorsque la batterie passe en dessous de 5,7 V, 
*	le moteur est coup. (GP0 = OFF). 
*	Une remise ON du moteur est possible (coup de manche mini+maxi) pour tirer le
* 	maximum des accus ...
*	Si le BEC n'est pas cabl, GP3 doit tre raccord au +5V.
*
*	Il faut noter que la fonction BEC n'a pas d'effet sur la sortie servo tout ou rien (GP1)
*
*	Le montage peut aussi servir d'inverseur de sens de rotation de servo.
*	le signal sur GP4 est l'inverse du signal servo en entree. L'inverseur est effectif
*	lorsque l' initialisation du neutre a t effectue. (3 secondes aprs mise ON)
*	
*
*
*******************************************************************************************/

typedef union			/* utilis pour grer les dbordements du timer */
{
	struct 
	{
		unsigned char low;
		unsigned char high;
	} n_cpt ;
	int cpt ;
} compteur;

//function prototypes
void interrupt isr(void);

// copyright en EEPROM
#asm	// Ecriture en EEPROM :
	psect	eedata,delta=2,abs,ovrld
	org		2100h
						// adresse 0 = mode (AUTO = 1, 2S = 2, 3S = 3)
	db		'A','E','R','O','P','I','C',' '
	db		'V','O','I','E','6','7','5',' '
	db		'V','3','.','0',' ','(','C',')'
	db		'2','0','0','7',' ','(','C',')'
	
			

	// Ecriture de la valeur OSCCAL en Flash :
	org		3FFh			// sert pour le debug MPLAB, A ENLEVER EN FINAL !!!
	dw		349Ch

#endasm


/* pas d'appels imbriqus ==> on peut passer par le stack en "fastcall" */
fastcall void						
Entrees (compteur *servo1)
{	
	do {					/* attente dbut impulsion */
	} while (E_emetteur == 0);

	/* ici impulsion servo ==> rinit variables */
	
	TMR0 = 155;				/* reset timer */
	do {					/* attente talon 100*8 usec */
	} while (TMR0);
	
	do {
	} while (E_emetteur == 1);			/* attente fin impulsion */
	(*servo1).cpt = TMR0 + 100;	/* rajout suite TMR0 */	
}

/* pas d'appels imbriqus ==> on peut passer par le stack en "fastcall" */
fastcall void						
Sorties (compteur consigne1)
{
	if (consigne1.n_cpt.low)
		{
			TMR0 =  (0xFF - consigne1.n_cpt.low) +1;	
			do {		
			} while (TMR0);
		}
	while ( consigne1.n_cpt.high--) 
	{
		TMR0 = 1;
		do {		
		} while (TMR0);
	} 
	

}					


/***********************/
/* programme principal */
/***********************/

main (void)
{
	unsigned char j;
	unsigned char i;
	unsigned char count;
	unsigned char step;
	int delta;

	int delta_min;		// seuil de declenchement bas
	int delta_max;		// seuil de declenchement haut
	int autorise;		// variable d'autorisation de mise ON/OFF
	int filtre_BEC;		// filtre de BEC

	compteur consigne;
	compteur neutreA;
	compteur duree_ESA;
	
	
	// INITIALISATION
	// ==============
	
	OSCCAL = _READ_OSCCAL_DATA();
	TRISIO	= 0b101000 ; 		/* GPIO3 et GPIO5 en entre */
	CMCON = 0b00000111;     //init du comparateur : general IO
	ANSEL = 0x00;           //turn off analog inputs
	ADON = 0x00;            //turn off A/D
	
	
							
	GPIO = 0x00;            //reset GPIO
	// OPTION = GPPU | !T0CS | T0SE | !PSA | 0x02; attention cette ligne ne passe pas
	OPTION = 0b10000010;						// oblig de passer au binaire !
		 /* Pull up disabled
			Internal cycle clock
			High to low source edge
			Prescaler assigned to timer
			Prescaler 1:8 */
	
	
	
	GPIE = 0 ; 			// disable port change interrupt
	IOCB = 0b000000 ; 	// inhibe les IT sur ports en entre
	GIE = 0; 			// global interrupt disable

	

	
	
	
	neutreA.cpt = 187;		/* dure neutre 1,5 ms soit 187,5 ticks */


/* calcul du neutre */
/********************/
	S_LED = ON;			/* LED ON */
	
	i = 3;				/* boucle 4 secondes filtre le neutre */
	do {						
		j = 50;			/* boucle 50 Hz */
		do { 
			duree_ESA.cpt = 0;
			Entrees (&duree_ESA);
			
			/* calcul et filtre du neutre */
			
			neutreA.cpt = neutreA.cpt + duree_ESA.cpt;
			neutreA.cpt = neutreA.cpt>>1;	
								
		} while (--j);

	} while (--i);
	

	
	delta_min = -15;
	delta_max = +15;
	delta = 15;
	step = 0;
	
/* calcul des seuils de declenchement min et max */
/*************************************************/
deb_calib :	
	
	Entrees (&duree_ESA);

// calcul du seuil min 

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
	
	
	if (consigne.cpt > -delta)  	// le manche n'a pas t abaiss 
	{
		goto fin;
	}	
	count = 0;
	step = 1;
	
deb_min :	
	
	Entrees (&duree_ESA);

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
		

	if (consigne.cpt < delta_min)	// le manche est plein piqu on cherche le minimum
	{
		delta_min = consigne.cpt;	
	}
	count++;
	if (count < 50)	// on attend la valeur min pendant 50 pulses soit 1 sec 
	{
		goto fin;
	}
	delta_min = delta_min+5; // on garde des marges ... 
	
	step = 2;
	
deb_calmax :
	
	
	Entrees (&duree_ESA);

// calcul du seuil max 

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
	
	
	if (consigne.cpt < delta)  	// le manche n'a pas t relev 
	{
		goto fin;
	}	
	count = 0;
	step = 3;
	
deb_max :	
	
	Entrees (&duree_ESA);

	consigne.cpt = duree_ESA.cpt - neutreA.cpt;
	
	

	if (consigne.cpt > delta_max)	// le manche est plein cabr 
	{
		delta_max = consigne.cpt;	
	}
	count++;
	if (count < 50)	// on attend la valeur max pendant 1 sec 
	{
		goto fin;
	}
	delta_max = delta_max-5; // on garde des marges ... 
	step = 4;	
	
	
/* boucle principale  */
/**********************/
	S_LED = OFF; 			/* fin de initialisations */
	i = 0;
	j = 0;
	autorise = 0;
	filtre_BEC = 0;
	delta = delta_min - 5 ; // pour que la voie vario tout ou rien soit init  0
	
debut :
	
	Entrees (&duree_ESA);

/* calcul de la consigne */
	
	consigne.cpt = duree_ESA.cpt - neutreA.cpt;

	if (consigne.cpt <= delta_min)	/* gestion manche bas */
	{
		j = 0;
		i++;
		if (i == 6)	// filtre autorisation on est reste 100ms au min
		{
			i = 0;
			autorise = 1;
		}
		
		goto fin;	/* attente prochaine impulsion */	
	}

	if (consigne.cpt < delta_max)	/* gestion resets */
	{
		i = 0;
		j ++;
		if (j == 16) 	// on n'est pas pass assez vite de min  max : plus de 300ms
		{
			j = 0;
			autorise = 0;
		}
		
		goto fin;	// attente prochaine impulsion 
	}
	
	if (consigne.cpt >= delta_max)	// gestion mise ON/off du moteur 
	{
		if (autorise == 1)	// il faut changer le moteur d'etat
		{
			S_moteur = !S_moteur;
			autorise = 0;
			// basculement de la consigne pour la voie servo tout ou rien (stock ds delta qui ne servait plus  rien)
			if (delta > 0) delta = delta_min - 5;
			else delta = delta_max + 5;
		}
		
		i = 0;
		j = 0;
		
		
		//goto fin;	// attente prochaine impulsion 
	}

fin :
	// gestion BEC
		if (E_BEC == Detecte)	// on a detecte une tension bat faible
			{
				filtre_BEC++;
				if (filtre_BEC == 20)	// il faut couper le jus
				{
					S_moteur = OFF;
					filtre_BEC = 0;
				}
			}
		else filtre_BEC = 0;
		
		
	// generation voie inverse
	duree_ESA.cpt = neutreA.cpt - consigne.cpt;
	/* appliquer la consigne inverse sur sortie */
	S_inverse = 1;
	Sorties (duree_ESA);
	S_inverse = 0;	
	
	switch (step) {
		case 0:	
 			goto deb_calib;
 	 		break;
 	 	case 1:	
 			goto deb_min;
 	 		break;
		case 2:
 			goto deb_calmax;		
			break;
		case 3:
 			goto deb_max;		
			break;
		
		
		default : step = 4;
		}
	
	
	// generation voie servo tout ou rien uniquement lorsque les initialisations sont faites
	duree_ESA.cpt = neutreA.cpt + delta;
	/* appliquer la consigne servo sur sortie */
	S_servo = 1;
	Sorties (duree_ESA);
	S_servo = 0;	
	
	do {
	} while (E_emetteur == 1);	/* attente fin impulsion servo le cas chant */
	goto debut;
}


/*******************interrupt service routines*****************/
void interrupt isr(void)
{
	
	GIE = 0;                //disable interrupts & return main	
}

/*******************************the end **************************/
